/*
 * ipcountry.c:
 * IP address to country lookup.
 *
 * Copyright (c) 2003 Chris Lightfoot. All rights reserved.
 * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
 *
 */

static const char rcsid[] = "$Id: ipcountry.c,v 1.3 2003/07/14 11:19:30 chris Exp $";

#define DATAFILE    "/software/share/ip-to-country/ip-to-country.dat"
#define RECORDLEN   10      /* 2 * sizeof(unit32_t) + 2 * sizeof(char) */

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include "countrycodes.h"

void read_rec(FILE *fp, const long index, uint32_t *lo, uint32_t *hi, char *cc) {
    char buf[10];
    fseek(fp, index * RECORDLEN, SEEK_SET);
    fread(buf, RECORDLEN, 1, fp);
    memcpy(lo, buf, sizeof *lo);
    memcpy(hi, buf + 4, sizeof *hi);
    if (cc) {
        memcpy(cc, buf + 8, 2);
        cc[2] = 0;
    }
}

int main(int argc, char *argv[]) {
    struct in_addr in;
    uint32_t n, ilo, ihi;
    FILE *fp;
    long olo, ohi, nrecs;
    char cc[3];
    int printname = 0;
    
    if (argc == 3 && strcmp(argv[1], "-n") == 0) {
        printname = 1;
        ++argv;
    } else if (argc != 2) {
        fprintf(stderr, "ipcountry: syntax is ipcountry [-n] ip-address\n");
        return 1;
    }

    if (!inet_aton(argv[1], &in)) {
        fprintf(stderr, "ipcountry: `%s' is not a valid IP address\n", argv[1]);
        return 1;
    }

    if (!(fp = fopen(DATAFILE, "rt"))) {
        perror("ipcountry: " DATAFILE);
        return 1;
    }
    
    fseek(fp, 0, SEEK_END);
    nrecs = ftell(fp) / RECORDLEN;
    fseek(fp, 0, SEEK_SET);

    olo = 0;
    ohi = nrecs - 1;
    
    n = (uint32_t)ntohl(in.s_addr);

    do {
        long o;

        o = (ohi + olo) / 2;
        read_rec(fp, o, &ilo, &ihi, cc);
        
        if (n < ilo)
            ohi = o;
        else if (n > ihi)
            olo = o;
        else
            break;
    } while (ohi > olo + 1);

    for (; olo <= ohi; ++olo) {
        read_rec(fp, olo, &ilo, &ihi, cc);
        if (n >= ilo && n <= ihi) {
            printf("%s", cc);
            if (printname) {
                printf(";%s", countryname(cc));
            }
            printf("\n");
            return 0;
        }
    }
    return 1;
}
